<?php
/**
 * CaptchaGenerator
 *
 * @author gkrsteski 03.09.2010 <gjero@krsteski.de>
 *
 * <samp>
 * try
 * {
 *     $captcha = new CaptchaGenerator(
 *                      'path/to/your/font.ttf',
 *                      'path/to/your/background-image.gif',
 *                      $the_length_of_your_phrase = 4);
 *
 *     // Intervene here the phrase.
 *     $the_captcha_phrase = $captcha->getPhrase();
 *
 *     $captcha->render();
 * }
 * catch (Exception $e)
 * {
 *     print $e->getMessage();
 * }
 * </samp>
 *
 */
final class CaptchaGenerator
{
    const FONT_COLOR_BLACK = 'black';
    const FONT_COLOR_WHITE = 'white';
    const FONT_COLOR_BLUE  = 'blue';

    /**
     * The background-image.
     * @var string
     */
    private $image = 'pwdimage.png';

    /**
     * The font-file.
     * @var string
     */
    private $font = 'arial.ttf';

    /**
     * The length of the secure-phrase.
     * @var integer
     */
    private $phraselength = 5;

    /**
     * The secure-phrase.
     * @var string
     */
    private $phrase = '';

    /**
     * The font-size.
     * @var integer
     */
    private $fontsize = 17;

    /**
     * The font-x-margin to the background-image.
     * @var integer
     */
    private $fontxmargin = 55;

    /**
     * The font-y-margin to the background-image.
     * @var integer
     */
    private $fontymargin = 4;

    /**
     * The font-color.
     * @var string
     */
    private $fontcolor = self::FONT_COLOR_BLUE;

    /**
     * The construct.
     * @param string $font
     * @param string $image
     * @param integer $phraselength
     * @return void
     */
    public function __construct($font = '', $image = '', $phraselength = '')
    {
        $this->setFont($font);
        $this->setImage($image);
        $this->setPhraselength($phraselength);

        $this->phrase = $this->getRandomPhrase();
    } // function

    /**
     * Sets the background-image.
     * @param string $image
     * @return CaptchaGenerator
     */
    public function setImage($image = '')
    {
        if(false === file_exists($image))
        {
            throw new Exception(
                'The background-image do not exists!');
        }

        $this->image = $image;
        return $this;
    } // function

    /**
     * Sets the font-file.
     * @param string $font
     * @return CaptchaGenerator
     */
    public function setFont($font = '')
    {
        if(false === file_exists($font))
        {
            throw new Exception(
                'The font do not exists!');
        }

        $this->font = $font;
        return $this;
    } // function

    /**
     * Sets the phrase-length.
     * @param integer $phraselength
     * @return CaptchaGenerator
     */
    public function setPhraselength($phraselength = 0)
    {
        if (false === is_int($phraselength) || $phraselength < 4)
        {
            throw new Exception(
                'The phrase-length must be an integer and bigger than 3!');
        }

        $this->phraselength = $phraselength;
        return $this;
    } // function

    /**
     * Sets the font-x-margin to the background-image.
     * @param integer $fontxmargin
     * @return CaptchaGenerator
     */
    public function setFontxmargin ($fontxmargin)
    {
        if (false === is_int($fontxmargin))
        {
            throw new Exception(
                'The font-x-margin must be an integer!');
        }

        $this->fontxmargin = $fontxmargin;
        return $this;
    } // function

    /**
     * Sets the font-y-margin to the background-image.
     * @param integer $fontymargin
     * @return CaptchaGenerator
     */
    public function setFontymargin ($fontymargin)
    {
        if (false === is_int($fontymargin))
        {
            throw new Exception(
                'The font-y-margin must be an integer!');
        }

        $this->fontymargin = $fontymargin;
        return $this;
    } // function

    /**
     * Sets the font-size.
     * @param integer $fontsize
     * @return CaptchaGenerator
     */
    public function setFontsize ($fontsize)
    {
        if (false === is_int($fontsize))
        {
            throw new Exception(
                'The font-size must be an integer!');
        }

        $this->fontsize = $fontsize;
        return $this;
    } // function

    /**
     * Sets the font-color.
     * @param string $fontcolor Expects blue | white | black.
     * @return CaptchaGenerator
     */
    public function setFontcolor($fontcolor)
    {
        $this->fontcolor = $fontcolor;
        return $this;
    } // function

    /**
     * Returns the phrase.
     * @return string
     */
    public function getPhrase()
    {
        return $this->phrase;
    } // function

    /**
     * Retrieves an random-phrase.
     * @return string
     */
    private function getRandomphrase()
    {
        $string = "27893456qwertzupasdfghjkyxcvbnm";
        $cout   = "";
        $pos    = 0;

        for ($i = 1; $i <= $this->phraselength; $i++)
        {
            $pos = rand(0, mb_strlen($string)-1);
            $cout .= $string{$pos};
        }

        return $cout;
    } // function

    /**
     * Generates the random-phrase with the
     * background-image and sends it as header-output.
     * @return void
     */
    public function render()
    {
        $src_id = 0;
        $a_img   = getimagesize($this->image);

        switch ($a_img['mime'])
        {
            case 'image/png'  : $src_id = imagecreatefrompng($this->image);  break;
            case 'image/jpeg' : $src_id = imagecreatefromjpeg($this->image); break;
            case 'image/gif'  : $old_id = imagecreatefromgif($this->image);
                                    $src_id = imagecreatetruecolor($a_img[0],$a_img[1]);
                                    imagecopy($src_id,$old_id,0,0,0,0,$a_img[0],$a_img[1]); break;
            default : break;
        }

        switch ($this->fontcolor)
        {
            case self::FONT_COLOR_BLACK : $fontcolor = imagecolorallocate($src_id, 0, 0, 0); break;
            case self::FONT_COLOR_WHITE : $fontcolor = imagecolorallocate($src_id, 255, 255, 255); break;
            case self::FONT_COLOR_BLUE : $fontcolor = imagecolorallocate($src_id, 0, 76, 134); break;
            default : case self::FONT_COLOR_BLUE : $fontcolor = imagecolorallocate($src_id, 0, 76, 134); break;
        }

        $x_size = imagesx($src_id);
        $y_size = imagesy($src_id);

        imagettftext(
            $src_id,
            $this->fontsize,
            0,
            $x_size - $this->fontxmargin,
            $y_size - $this->fontymargin,
            $fontcolor,
            $this->font,
            $this->phrase
        );

        switch($a_img['mime'])
        {
            case 'image/png'  : Header("Content-type: image/png"); imagepng($src_id); break;
            case 'image/jpeg' : Header("Content-type: image/jpeg"); imagejpeg($src_id); break;
            case 'image/gif'  : Header("Content-type: image/gif"); imagegif($src_id); break;
            default : break;
        }

        imagedestroy($src_id);
    } // function
} // class